/** 
 * @(#)mod_iasp.c 
 * copyright(c) Halcyon Software 2000 
 * 
 */ 
#include "httpd.h"
#include "mod_iasp.h"
#include "utils.h"
#include "http_conf_globals.h" 

/* Forward declaration for module object. */
//#include "crtdbg.h"
#ifdef WIN32
MODULE_VAR_EXPORT module iasp_module;
#else
module iasp_module;
#endif

tcpPool				*tcp_pool;		/* pool of connection sockets */
//Cluster_tcp_pool	*cluster_tcp_pool = NULL; it is for cluster server, so removed it, 8/5/2000


void iasp_log_msg(server_rec *s, int type, char *format, ...) {

    va_list lst;
    char msg[1024];

    va_start (lst, format);
    vsprintf (msg, format, lst);
    va_end (lst);

#if MODULE_MAGIC_NUMBER >= 19980413
    ap_log_error(APLOG_MARK, type, s, "%s", msg);
#else
    log_error(msg, s);
#endif

}

static void iasp_child_init(server_rec *s, pool *p) {

   mod_config* c = get_module_config( s->module_config, &iasp_module );

	tcp_pool = init_pool(c->host, c->port, c->pool_size, c->max_pool_size, c->timeout);

#ifdef _DEBUG
	iasp_log_msg(s, APLOG_NOERRNO | APLOG_NOTICE, "mod_iasp: iasp_child_init called");
#endif

}

static void iasp_child_exit(server_rec *s, pool *p) {

    mod_config* c = get_module_config( s->module_config, &iasp_module );
	destroy_tcpPool(tcp_pool);
#ifdef _DEBUG
	iasp_log_msg(s, APLOG_NOERRNO | APLOG_NOTICE, "mod_iasp: iasp_child_exit called");
#endif

}

static void* create_iasp_conf( pool *p, server_rec *s ) {
	mod_config	*c		= (mod_config*) pcalloc( p, sizeof(mod_config) );
    c->maps 			= make_array(p, 256, sizeof(char *));
	c->host				= ap_pstrdup(p, "127.0.0.1"); 
	c->pool_size		= 1;
#ifdef WIN32
	c->max_pool_size	= 256;
#else
	c->max_pool_size    = 1;
#endif

	c->timeout			= 30;	


   return (void*) c;
}


const char *get_table_value(Table *props, const char *prop_name) {

	int j;
	for (j=0; j< props->count; j++) {
		char *name = props->entry[j].key;
		char *value = props->entry[j].val;
		if (strcasecmp(name, prop_name) == 0) {
			return value;
		}
	}

	return NULL;
}

//add virtualhost by qinbing
/*
int findHost(struct v_host *vhost_list, char * var) {
	 struct v_host *temp;
	   if(vhost_list == NULL)  return 0;
	   temp = vhost_list;
		while(temp->virtual_host != NULL){

			if(strcasecmp(temp->virtual_host,var)==0) {
  			 return 1;
			}
			temp = temp->next;
		}
	return 0;
}

void addhostname(char* var) {
	struct v_host *temp;
	if(findHost(vhost_list,var)) {
       return ; 
	}
	if(vhost_list == NULL) {
		vhost_list = (struct v_host *)malloc(sizeof(struct v_host));
		vhost_list->virtual_host = (char *)malloc(strlen(var)+1); 
		vhost_list->next = NULL;
		strcpy(vhost_list->virtual_host,var);
		return ;
	}
	
	temp = vhost_list;
	while(temp->next != NULL) {
       temp= temp->next;     
	}

	temp->next = (struct v_host *)malloc(sizeof(struct v_host));
	temp->next->virtual_host = (char *)malloc(strlen(var)+1); 
	strcpy(temp->next->virtual_host,var);
	//free(temp);
}


const char * iasp_virtualhost( cmd_parms* parms, void* dummy, char* var, char* val ) {
	addhostname(var);
	return NULL;
}
*/
const char * iasp_config( cmd_parms* parms, void* dummy, char* var, char* val ) {

    server_rec	*s = parms->server;
    mod_config	*c = get_module_config( s->module_config, &iasp_module );
	int			 i = 0;

	if (strcasecmp(var,"server") == 0) {

		Table *props = get_table(val);

		if (props != NULL) {
			int j;
			for(j=0;j<props->count;j++) {

				char *name  = props->entry[j].key;
				char *value = props->entry[j].val; 

				if ( strcasecmp( name, "ProxyHost" ) == 0 ) {

					c->host = ap_pstrdup(parms->pool, value);

            } else if ( strcasecmp( name, "ProxyPort" ) == 0 )
					c->port = atoi(value);
				else if ( strcasecmp( name, "Timeout" ) == 0 )
					c->timeout = atoi(value);
#ifdef WIN32
				/* Connection pooling does makes sense for now only on Win32,
				   because on Win32 Apache handles requests in separate threads.
				   On UNIX each request is handled by one child process that uses
				   only one connection */
				else if ( strcasecmp( name, "InitPoolSize" ) == 0 )
					c->pool_size = atoi(value);
				else if ( strcasecmp( name, "MaxPoolSize" ) == 0 )
					c->max_pool_size = atoi(value);
#endif
			    /*it is for cluster server, so remove it, 8/5/2000
				else if ( (strstr(name,"ClusterEnable") != NULL) && (strstr(value,"true") != NULL)) {

					#ifdef _DEBUG
					iasp_log_msg(parms->server,APLOG_NOERRNO|APLOG_NOTICE,"cluster!!");					
					#endif
					init_aul_tcp_pool(props,c,parms);

				}*/

			}

#ifdef _DEBUG					
			iasp_log_msg(parms->server, APLOG_NOERRNO | APLOG_NOTICE, "mod_iasp:  settings: \
ProxyHost = %s, ProxyPort = %d, Timeout = %d, InitPoolSize = %d, MaxPoolSize = %d", \
				c->host, c->port, c->timeout, c->pool_size, c->max_pool_size);
#endif

		}

		free(props);

	} else if ( strcasecmp( var, "rules" ) == 0 )	{

		Table *props = get_table(val);

		if(props != NULL) {
			int j;
			for(j=0;j<props->count;j++) {
				char *name = props->entry[j].key;
				if(strstr(name, "serversync.") == NULL && strstr(name, "synchandler.") == NULL){
					char **map = (char **) push_array(c->maps);
					*map = ap_pstrdup(parms->pool, name);
#ifdef _DEBUG					
					iasp_log_msg(parms->server, APLOG_NOERRNO | APLOG_NOTICE, "mod_iasp:  mappings: %s", *map);
#endif									
				}
			}
		}

		free(props);
	}
#ifdef _DEBUG					
	iasp_log_msg(parms->server, APLOG_NOERRNO | APLOG_NOTICE, "mod_iasp:  iasp_config() called");
#endif

    return (char*) 0;
}

#ifndef WIN32
int IsCharAlphaNumeric(char c) {

	if(isalpha(c) || isdigit(c))
		return 1;
	else
		return 0;
}
/*
char *itostring(char *name, int port){
	  char *result;
	  asprintf(&result, "%s:%d", name, port);
	  return result;
	}
	*/
#endif

static int iasp_trans(request_rec * r) {

	int			j;
	mod_config	*c;
	char		lastMap[1024];
	char		*mapping    = NULL;
	char		*req		= (char *) palloc(r->pool, sizeof(char) * (strlen(r->uri) + 1));
	const char	*busyflag	= table_get(r->subprocess_env, "request_busy");
/*	char		hostname[100];
	char		*colon;
	char		numport[25];
	unsigned    port;
	struct v_host *temp;
	
	colon = ":";
	strcpy(hostname,r->connection->remote_ip);
	port = r->server->addrs->host_port;

#ifdef WIN32
	itoa(port,numport,10);

	if (port){
		strcat(hostname,colon);
		strcat(hostname,numport);
	}
#else
	if(port){
		strcpy(hostname,itostring(hostname,port));
	}

#endif

	if(r->server->is_virtual==1){
		temp = vhost_list;
		while(temp != NULL){
			if(strcasecmp(temp->virtual_host,hostname)==0) {
			    return DECLINED;
			}
			temp = temp->next;
		}
	}*/ 
	
	if(req == NULL) {
		return DECLINED;
	}

	if(busyflag != NULL && strcmp(busyflag, "true") == 0) {
		return DECLINED;
	}

	c = get_module_config( r->server->module_config, &iasp_module );

	strcpy(lastMap, "");
	strcpy(req, r->uri);
	

	for(j=0;j < c->maps->nelts;j++) {

		char *map = ((char **) (c->maps->elts))[j];

		if(map[0] == '*') {

			char * str;
			char * tem;
			char * newmap = &map[1];
			
			if((str = strstr(req, ".")) == NULL) 	continue;

			tem = str;
			while(tem != NULL) {
				tem++;
				if(tem == NULL || IsCharAlphaNumeric(*tem) == FALSE) {
					*tem = '\0';
					break;
				}
			}

			#if defined(WIN32) || defined(OS2) || defined(NetWare)
				if(stricmp(str, newmap) == 0) 
			#else
				if(strcmp(str, newmap) == 0) 
			#endif
			{
				r->handler = pstrdup(r->pool, "iasp-proxy");
				return DECLINED;
			}
			
		}
	}

	for(j=0;j <  c->maps->nelts;j++) {

		char *map = ((char **) (c->maps->elts))[j];

		if(map[0] != '*') {

			if(map[strlen(map)-1] == '*')
				map[strlen(map)-1] = '\0';
			#if defined(WIN32) || defined(OS2) || defined(NetWare)
				if(strnicmp(map, req, strlen(map)) == 0) 
			#else
				if(strncmp(map, req, strlen(map)) == 0) 
			#endif
			{
				if(mapping == NULL || strlen(map) > strlen(lastMap)) {
					strcpy(lastMap, map);
					mapping = map;
				}

			}

		}

	}

	if(strlen(lastMap) > 0) {
		r->handler = pstrdup(r->pool, "iasp-proxy");
		return DECLINED;
	} else
		return DECLINED;

}

static int iasp_service( request_rec* req ) {
	
       sockState	*conn  = NULL;
	server_rec	*s     = req->server;

	mod_config	*c     = get_module_config( s->module_config, &iasp_module );
    char  *localip     =NULL; 
   tcpPool     *pool  = NULL;
	//int         count  = 1;  it is for cluster server, so removed it, 8/5/2000
	//server_info *info  = NULL; it is for cluster server, so removed it, 8/5/2000
	int         retFlag= 1;
	int			port		= 9098;
	char        *host_name	= NULL;
	char        *session	= NULL;
	int         index		= -1;
    int         trynumber=1;  //the number of trying to connect iASP proxy server.
    const char *auth_line=NULL;
	const char *t=NULL;
	const char *pw=NULL;

#if MODULE_MAGIC_NUMBER < 19980413
	if (tcp_pool == NULL)
		tcp_pool = init_pool(c->host, c->port, c->pool_size, c->max_pool_size, c->timeout);
#endif

	add_cgi_vars(req);
	add_common_vars(req);

pool  = tcp_pool;
#ifdef _DEBUG
    iasp_log_msg(s,APLOG_NOERRNO|APLOG_NOTICE,pool->host);
#endif

	
	if ((conn = get_connection(pool, pool->host, pool->port)) == NULL) {
		char	*str = "HTTP/1.0 200 OK\r\nContent-type: text/html\r\n\r\n<H2>Could not connect to iasp Connector Proxy</H2><H3>Please contact the system administrator for this web server </H3>";
		rwrite(str, strlen(str), req);
		return OK;
	}
 	localip=ap_pstrdup(req->connection->pool,inet_ntoa(req->connection->local_addr.sin_addr));

	table_set(req->subprocess_env,"LOCAL_ADDR",localip);

	table_set(req->subprocess_env, "URL", req->uri);

	table_set(req->subprocess_env, "request_busy", "true");
	
	if (strcasecmp(http_method(req),"https") == 0) {
        table_set(req->subprocess_env, "SERVER_PORT_SECURE", "1");
		table_set(req->subprocess_env,"HTTPS","on");
	} else {
		table_set(req->subprocess_env, "SERVER_PORT_SECURE", "0");
        table_set(req->subprocess_env,"HTTPS","off");
	}

    if(req->headers_in)
       auth_line = ap_table_get(req->headers_in,
                                      req->proxyreq ? "Proxy-Authorization"
                                                  : "Authorization");
    if (auth_line!=NULL) {

	    if (strcasecmp(ap_getword(req->pool, &auth_line, ' '), "Basic")) {
           ap_log_rerror(APLOG_MARK, APLOG_NOERRNO|APLOG_ERR, req,
                       "client used wrong authentication scheme: %s", req->uri);
           ap_note_basic_auth_failure(req);
           return AUTH_REQUIRED;
       }
    
       t = ap_uudecode(req->pool, auth_line);
       req->connection->user = ap_getword_nulls (req->connection->pool, &t, ':');
	   
       pw = t;
	    table_set(req->subprocess_env, "LOGON_USER", req->connection->user);
	    table_set(req->subprocess_env, "AUTH_TYPE", ap_auth_type(req));
	    table_set(req->subprocess_env, "AUTH_PASSWORD", pw);
	    table_set(req->subprocess_env, "AUTH_USER", req->connection->user);
	    //print_header(req->headers_in);
	    //print_table(req->headers_in);
    }

Label:

	retFlag = handle_request(conn,req,pool);

	/*if (retFlag == 401) {
       ap_table_setn(req->err_headers_out,  req->proxyreq ? "Proxy-Authenticate" : "WWW-Authenticate",
                  ap_pstrcat(req->pool, "Basic realm=\"", NULL, "\"",
                       NULL));
		return AUTH_REQUIRED;
	}*/


#ifdef _DEBUG
	iasp_log_msg(s, APLOG_NOERRNO | APLOG_NOTICE, "handle_request.returnValue=%d\n", retFlag);
#endif


	if (retFlag == -1){

		/*it is for cluster server, so removed it, 8/5/2000
        #ifdef WIN32
		if (info != NULL) {
			info->max_handles--;
		}
		#endif*/
		return 408; 

   } else if(retFlag == 0) {
		if(trynumber < 2) {
			if ((conn = get_connection(pool, pool->host, pool->port)) != NULL) {
				   trynumber++;
				   goto Label;
			}
		}else{
             return HTTP_INTERNAL_SERVER_ERROR;
		}
	}

   /*it is for cluster server, so removed it, 8/5/2000	
   if (info != NULL) {
		;
		#ifdef WIN32
		info->max_handles--;
		#endif
	}*/

	conn->avail = TRUE;	
	return OK;

}

command_rec iasp_commands [] = {
//	{ "ForbidiASPHost", iasp_virtualhost, NULL, ACCESS_CONF|RSRC_CONF, TAKE1, "a IaspVirtualhost to set followed by its value" },
	{ "IaspConfig", iasp_config, NULL, ACCESS_CONF|RSRC_CONF, TAKE2, "a variable to set followed by its value" },
	{ NULL }
};

handler_rec iasp_handlers[] ={
   { "iasp-proxy", iasp_service }, 
   { NULL }
};

#ifdef WIN32
	module MODULE_VAR_EXPORT iasp_module =
#else
	module iasp_module =
#endif
{
    STANDARD_MODULE_STUFF,
    NULL,					/* initializer */
    NULL, 					/* dir config creater */
    NULL,					/* dir merger - default is to override */
    create_iasp_conf,	    /* server config */
    NULL,				    /* merge server config */
    iasp_commands,		    /* command table */
    iasp_handlers,		    /* handlers */
    iasp_trans,			    /* filename translation */
    NULL,					/* check_user_id */
    NULL,					/* check auth */
    NULL,					/* check access */
    NULL,					/* type_checker */
    NULL,					/* fixups */
    NULL,					/* [10] logger */
    NULL,					/* [3] header parser */
#if MODULE_MAGIC_NUMBER >= 19980413
    iasp_child_init,		/* process initializer */
    iasp_child_exit,		/* process exit/cleanup */
    NULL					/* [1] post read_request handling */
#endif
};