/** 
 * @(#)utils.h 
 * copyright(c) Halcyon Software 2000 
 * 
 */ 

#ifndef UTILS_H
#define UTILS_H

#if defined __cplusplus
extern "C" {
#endif

#ifdef WIN32

#include <winsock.h>
#ifndef EINTR
	#define EINTR WSAEINTR
#endif 

#ifndef ETIMEDOUT
	#define ETIMEDOUT WSAETIMEDOUT
#endif 

#ifndef sleep
	#define sleep(t) Sleep(t * 1000)
#endif 

#define getErrno() WSAGetLastError()

#else 

	#include <arpa/inet.h>
	#include <errno.h>	
	#include "httpd.h"
	#include <netdb.h>
	#include <netinet/in.h>	
	#include <netinet/tcp.h>	
	#include <sys/socket.h>	
	#include <sys/types.h>
	#include <unistd.h>

	#define BOOL short
	#define TRUE 1
	#define FALSE 0

	#define closesocket(s) close(s)
	#define getErrno() errno
	#define wsprintf sprintf

#endif

#ifdef WIN32
	static CRITICAL_SECTION poolLock;
#elif defined(NSAPI) 
	#include <base/crit.h>
	#include <base/systems.h>
	static CRITICAL poolLock;
#endif 

#include <memory.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#define NOOP				0 
#define MAP					1
#define MAP_CHECK			2
#define GET_HEADER			3
#define GET_REALPATH		4
#define GET_POSTDATA		5
#define PUT_STATUS			6
#define PUT_HEADER			7
#define PUT_CLIENT			8
#define FINISH				9
#define DESTROYALL			10
#define DESTROYALL_QUIT		11
#define PING				12
#define GOOD                13
#define BUSY                14
#define UNKNOWN				100

typedef struct {

	int avail;
	int sock;
	int timeout;

} sockState;

typedef struct {
	
	int			max_size;
	int			size;
	int			timeout;
    int			port;				    
	char		*host;
	sockState	pool[1024];

} tcpPool;

typedef struct {

	int		 count;
	tcpPool *aul_tcp_pool[15];

} Cluster_tcp_pool;

typedef struct property {

	char key[1024];
	char val[2048];

} Prop;

typedef struct table {

	int		count;
	Prop		entry[64];

} Table;

Table	  *get_table(char *filename);

#ifdef WIN32
	typedef struct request_rec request_rec;
#endif

int			handle_request  (sockState *p,request_rec *r,tcpPool *pool);
char *		get_header      (char *name,request_rec *r);
char *		get_real_path   (request_rec *r, char *path);
char *		get_post_data	(request_rec *r, int *len);
void		write_header	(request_rec *r, char *szStatus, char *szHeaders);

tcpPool  *	init_pool(char *host, int port, int initSize, int maxSize, int timeout);
void		destroy_tcpPool(tcpPool *);
sockState * get_connection(tcpPool *pool, char *host, int port);
int			get_connection_socket(char *host, int port);
void		close_connection(int sock);

#if defined __cplusplus
}
#endif
#endif