/** 
 * @(#)mod_iasp.c 
 * copyright(c) Halcyon Software 2000 
 * 
 */ 
#include "httpd.h"
#include "mod_iasp.h"
#include "utils.h"
#include "apr_strings.h"

#ifdef WIN32
MODULE_VAR_EXPORT module iasp_module;
#else
module iasp_module;
#endif

tcpPool				*tcp_pool;		/* pool of connection sockets */

void iasp_log_msg(server_rec *s, int type, char *format, ...) {

    va_list lst;
    char msg[1024];
    va_start (lst, format);
	vsprintf (msg, format, lst);
    va_end (lst);
	
	ap_log_error(APLOG_MARK, type,0, s, "%s", msg);
}

/*
static void iasp_child_init(server_rec *s, apr_pool_t *p) { //pool
	
	mod_config* c = get_module_config( s->module_config, &iasp_module );
	tcp_pool = init_pool(c->host, c->port, c->pool_size, c->max_pool_size, c->timeout);

#ifdef _DEBUG
	iasp_log_msg(s, APLOG_NOERRNO | APLOG_NOTICE, "mod_iasp: iasp_child_init called");
#endif

}

static void iasp_child_exit(server_rec *s, apr_pool_t *p) { //pool
	
    mod_config* c = get_module_config( s->module_config, &iasp_module );
	
	destroy_tcpPool(tcp_pool);
#ifdef _DEBUG
	iasp_log_msg(s, APLOG_NOERRNO | APLOG_NOTICE, "mod_iasp: iasp_child_exit called");
#endif
}*/

static void* create_iasp_conf( apr_pool_t *p, server_rec *s ) { //pool
	mod_config	*c		= (mod_config*) ap_pcalloc( p, sizeof(mod_config) ); //pcalloc
    c->maps 			= ap_make_array(p, 256, sizeof(char *)); //make_array
	c->host				= ap_pstrdup(p, "127.0.0.1");
	
	c->pool_size		= 1;
#ifdef WIN32
	c->max_pool_size	= 256;
#else
	c->max_pool_size    = 1;
#endif

	c->timeout			= 30;	

    return (void*) c;
}

const char *get_table_value(Table *props, const char *prop_name) {

	int j;
	for (j=0; j< props->count; j++) {
		char *name = props->entry[j].key;
		char *value = props->entry[j].val;
		if (strcasecmp(name, prop_name) == 0) {
			return value;
		}
	}

	return NULL;
}

const char * iasp_config( cmd_parms* parms, void* dummy, char* var, char* val ){

	server_rec	*s = parms->server;
    mod_config	*c = ap_get_module_config( s->module_config, &iasp_module ); //get_module_config

	int			 i = 0;
	if (strcasecmp(var,"server") == 0) {

		Table *props = get_table(val);
		
		if (props != NULL) {
			int j;
			for(j=0;j<props->count;j++) {

				char *name  = props->entry[j].key;
				char *value = props->entry[j].val;
				
				if ( strcasecmp( name, "ProxyHost" ) == 0 )
					c->host = ap_pstrdup(parms->pool, value); //pstrdup
				else if ( strcasecmp( name, "ProxyPort" ) == 0 )
					c->port = atoi(value);
				else if ( strcasecmp( name, "Timeout" ) == 0 )
					c->timeout = atoi(value);
#ifdef WIN32
				/* Connection pooling does makes sense for now only on Win32,
				   because on Win32 Apache handles requests in separate threads.
				   On UNIX each request is handled by one child process that uses
				   only one connection */
				else if ( strcasecmp( name, "InitPoolSize" ) == 0 )
					c->pool_size = atoi(value);
				else if ( strcasecmp( name, "MaxPoolSize" ) == 0 )
					c->max_pool_size = atoi(value);
#endif
			}

#ifdef _DEBUG
			iasp_log_msg(parms->server, APLOG_NOERRNO | APLOG_NOTICE, "mod_iasp:  settings: \
ProxyHost = %s, ProxyPort = %d, Timeout = %d, InitPoolSize = %d, MaxPoolSize = %d", \
				c->host, c->port, c->timeout, c->pool_size, c->max_pool_size); /**************/
#endif
		}
		free(props);
	} else if ( strcasecmp( var, "rules" ) == 0 )	{

		Table *props = get_table(val);

		if(props != NULL) {
			int j;
			for(j=0;j<props->count;j++) {
				char *name = props->entry[j].key;
				if(strstr(name, "serversync.") == NULL && strstr(name, "synchandler.") == NULL){
					char **map = (char **) ap_push_array(c->maps); // ap_push_array
					*map = ap_pstrdup(parms->pool, name); 
#ifdef _DEBUG					
					iasp_log_msg(parms->server, APLOG_NOERRNO | APLOG_NOTICE, "mod_iasp:  mappings: %s", *map);/*************/
#endif									
				}
			}
		}

		free(props);
	}
#ifdef _DEBUG					
	iasp_log_msg(parms->server, APLOG_NOERRNO | APLOG_NOTICE, "mod_iasp:  iasp_config() called"); /**************/
#endif
	
	return (char*) 0;
}

//#ifndef WIN32
	int IsCharAlphaNumeric(char c) {

		if(isalpha(c) || isdigit(c))
			return 1;
		else
			return 0;
	}
//#endif

static int iasp_service( request_rec* req ) {
	
	sockState	*conn  = NULL;
	server_rec	*s     = req->server;
	
	mod_config	*c     = ap_get_module_config( s->module_config, &iasp_module ); //get_module_config
    char  *localip     =NULL; 
    tcpPool     *pool  = NULL;
	
	int         retFlag= 1;
	int			port		= 9098;
	char        *host_name	= NULL;
	char        *session	= NULL;
	int         index		= -1;
    int         trynumber=1;  //the number of trying to connect iASP proxy server.
    const char *auth_line=NULL;
	const char *t=NULL;
	const char *pw=NULL;
	
	if (tcp_pool == NULL)
		tcp_pool = init_pool(c->host, c->port, c->pool_size, c->max_pool_size, c->timeout);

	ap_add_cgi_vars(req); //add_cgi_vars
	ap_add_common_vars(req); //add_common_vars

pool  = tcp_pool;
#ifdef _DEBUG
    iasp_log_msg(s,APLOG_NOERRNO|APLOG_NOTICE,pool->host);
#endif

	if ((conn = get_connection(pool, pool->host, pool->port)) == NULL) {
		char	*str = "<H2>Could not connect to iasp Connector Proxy</H2><H3>Please contact the system administrator for this web server </H3>";
		
		req->status = 200;
		req->content_type = "text/html";
        ap_rwrite(str, strlen(str), req); //rwrite
		return OK;
	}
 	
	localip= ap_pstrdup(req->connection->pool,inet_ntoa(req->connection->local_addr->sa.sin.sin_addr));
	
	ap_table_set(req->subprocess_env,"LOCAL_ADDR",localip); //table_set
    
	ap_table_set(req->subprocess_env, "URL", req->uri);

	ap_table_set(req->subprocess_env, "request_busy", "true");
	
	if (strcasecmp(ap_http_method(req),"https") == 0) { //http_method
        ap_table_set(req->subprocess_env, "SERVER_PORT_SECURE", "1");
		ap_table_set(req->subprocess_env,"HTTPS","on");
	} else {
		ap_table_set(req->subprocess_env, "SERVER_PORT_SECURE", "0");
        ap_table_set(req->subprocess_env,"HTTPS","off");
	}

    if(req->headers_in)
       auth_line = ap_table_get(req->headers_in,
                                      req->proxyreq ? "Proxy-Authorization"
                                                  : "Authorization");
    if (auth_line!=NULL) {

	    if (strcasecmp(ap_getword(req->pool, &auth_line, ' '), "Basic")) {
           ap_log_rerror(APLOG_MARK, APLOG_NOERRNO|APLOG_ERR,0, req,
                       "client used wrong authentication scheme: %s", req->uri);
           ap_note_basic_auth_failure(req);
           return HTTP_UNAUTHORIZED; //AUTH_REQUIRED
       }
    
       t = ap_pbase64decode(req->pool, auth_line); // ap_uudecode
       //req->connection->user = ap_getword_nulls (req->connection->pool, &t, ':'); 
	   req->user = ap_getword_nulls(req->connection->pool, &t, ':');
	   
       pw = t;
	   ap_table_set(req->subprocess_env, "LOGON_USER", req->user);
	   ap_table_set(req->subprocess_env, "AUTH_TYPE", ap_auth_type(req));
	   ap_table_set(req->subprocess_env, "AUTH_PASSWORD", pw);
	   ap_table_set(req->subprocess_env, "AUTH_USER", req->user);
    }

Label:

	retFlag = handle_request(conn,req,pool);

	/*if (retFlag == 401) {
       ap_table_setn(req->err_headers_out,  req->proxyreq ? "Proxy-Authenticate" : "WWW-Authenticate",
                  ap_pstrcat(req->pool, "Basic realm=\"", NULL, "\"",
                       NULL));
		return AUTH_REQUIRED;
	}*/


#ifdef _DEBUG
	iasp_log_msg(s, APLOG_NOERRNO | APLOG_NOTICE, "handle_request.returnValue=%d\n", retFlag);
#endif


	if (retFlag == -1){
		return 408; 
    }else if(retFlag == 0) {
		if(trynumber < 2) {
			if ((conn = get_connection(pool, pool->host, pool->port)) != NULL) {
				   trynumber++;
				   goto Label;
			}
		}else{
             return HTTP_INTERNAL_SERVER_ERROR;
		}
	}

	conn->avail = TRUE;	
	return OK;
}

static int iasp_trans(request_rec * r) {

	int			j;
	mod_config	*c;
	char		lastMap[1024];
	char		*mapping    = NULL;
	char		*req		= (char *) ap_palloc(r->pool, sizeof(char) * (strlen(r->uri) + 1)); //palloc
	const char	*busyflag	= ap_table_get(r->subprocess_env, "request_busy"); //table_get
	
	if(req == NULL) {
		return DECLINED;
	}

	if(busyflag != NULL && strcmp(busyflag, "true") == 0) {
		return DECLINED;
	}
	
	c = ap_get_module_config( r->server->module_config, &iasp_module ); //get_module_config

	strcpy(lastMap, "");
	strcpy(req, r->uri);
	

	for(j=0;j < c->maps->nelts;j++) {

		char *map = ((char **) (c->maps->elts))[j];

		if(map[0] == '*') {

			char * str;
			char * tem;
			char * newmap = &map[1];
			
			if((str = strstr(req, ".")) == NULL) 	continue;

			tem = str;
			while(tem != NULL) {
				tem++;
				if(tem == NULL || IsCharAlphaNumeric(*tem) == FALSE) {
					*tem = '\0';
					break;
				}
			}

			#if defined(WIN32) || defined(OS2) || defined(NetWare)
				if(stricmp(str, newmap) == 0) 
			#else
				if(strcmp(str, newmap) == 0) 
			#endif
			{
				return iasp_service(r);
			}
			
		}
	}

	for(j=0;j <  c->maps->nelts;j++) {

		char *map = ((char **) (c->maps->elts))[j];

		if(map[0] != '*') {

			if(map[strlen(map)-1] == '*')
				map[strlen(map)-1] = '\0';
			#if defined(WIN32) || defined(OS2) || defined(NetWare)
				if(strnicmp(map, req, strlen(map)) == 0) 
			#else
				if(strncmp(map, req, strlen(map)) == 0) 
			#endif
			{
				if(mapping == NULL || strlen(map) > strlen(lastMap)) {
					strcpy(lastMap, map);
					mapping = map;
				}

			}

		}

	}

	if(strlen(lastMap) > 0) {
		return iasp_service(r);
	} else
		return DECLINED;
}

command_rec iasp_commands [] = {
	{ "IaspConfig", iasp_config, NULL, ACCESS_CONF|RSRC_CONF, TAKE2, "a variable to set followed by its value" },
	{ NULL }
};

static void iasp_register_hooks(apr_pool_t *cont)
{
	ap_hook_handler(iasp_trans, NULL, NULL, APR_HOOK_MIDDLE);
}

#ifdef WIN32
	module MODULE_VAR_EXPORT iasp_module =
#else
	module iasp_module =
#endif
{
	STANDARD20_MODULE_STUFF,
    NULL,    /* per-directory config creator */
    NULL,     /* dir config merger */
    create_iasp_conf, /* server config creator */
    NULL,  /* server config merger */
    iasp_commands, /* command table */
    iasp_register_hooks,       /* set up other request processing hooks */
};
