/** 
 * @(#)proxy.c 
 * copyright(c) Halcyon Software 2000 
 * 
 */ 

#include "mod_iasp.h"
#include "utils.h"

#if defined UNIXWARE
	#include <utmpx.h>
#endif

/*
#if defined LINUX
	#include <timebits.h>
#endif
*/

#ifdef AIX
	#include <sys/select.h>
#endif

char * HEADERNAMES[] =
{
"APPL_MD_PATH",
"APPL_PHYSICAL_PATH",
"AUTH_TYPE",
"AUTH_PASSWORD",   
"AUTH_USER",
"CERT_COOKIE",
"CERT_FLAGS",   
"CERT_ISSUER",
"CERT_KEYSIZE",   
"CERT_SECRETKEYSIZE",
"CERT_SERIALNUMBER",   
"CERT_SERVER_ISSUER",   
"CERT_SERVER_SUBJECT",   
"CERT_SUBJECT",   
"CONTENT_LENGTH",
"CONTENT_TYPE",
"GATEWAY_INTERFACE",
"HTTP_ACCEPT",
"HTTP_ACCEPT_ENCODING",
"HTTP_ACCEPT_LANGUAGE",
"HTTP_COOKIE",
"HTTP_CONNECTION",
"HTTP_HOST",
"HTTP_REFERER",
"HTTP_USER_AGENT",
"HTTPS",
"HTTPS_KEYSIZE",   
"HTTPS_SECRETKEYSIZE",   
"HTTPS_SERVER_ISSUER",   
"HTTPS_SERVER_SUBJECT",   
"INSTANCE_ID",
"INSTANCE_META_PATH",
"LOCAL_ADDR",
"LOGON_USER",   
"PATH_INFO",
"PATH_TRANSLATED",
"QUERY_STRING",   
"REMOTE_ADDR",
"REMOTE_HOST",
"REMOTE_USER",
"REQUEST_METHOD",
"REAL_PATH",
"SCRIPT_NAME",
"SERVER_NAME",
"SERVER_PORT",
"SERVER_PORT_SECURE",
"SERVER_PROTOCOL",
"SERVER_SOFTWARE",
"URL"
};

#define HEADERCOUNT (sizeof(HEADERNAMES) / sizeof(char *))

char *get_real_path(request_rec *r, char *path) {
   char *name_ptr = NULL;
   request_rec *pa_req = NULL;
   char	*value = NULL;
   int  i,j;

#if MODULE_MAGIC_NUMBER > 19990101
	name_ptr = ap_escape_uri(r->pool, path);
#else
	name_ptr = escape_uri(r->pool, path);
#endif 

	pa_req = ap_sub_req_lookup_uri(name_ptr, r,NULL); //sub_req_lookup_uri
	
	value  = (char *)ap_pstrcat(r->pool, pa_req->filename, pa_req->path_info, NULL); //pstrcat
  	
    ap_destroy_sub_req(pa_req); //destroy_sub_req
    if (value != NULL) {
    for(i=0 ;i<(int)strlen(value);i++) {
	    if ((value[i]=='/')&& (value[i+1]=='/')) {
		    for (j=i;j<(int)strlen(value)-1;j++,i++) {
			    value[i+1]=value[j+2];
			}
            value[i]='\0';
            break;  
		}
	}
	}
   
   return (value == NULL ? NULL : strdup(value));
}

char *get_header(char *name,request_rec *r){

	char	*value;
	

	if(strcmp(name, "REMOTE_HOST") == 0){
		name = "REMOTE_ADDR";
	}

	if (strcmp(name,"REAL_PATH") == 0) {
		apr_uri_t		    parseduri	= r->parsed_uri;	//uri_components
		value = get_real_path(r,parseduri.path);
	}else{											   	
		value = (char *) apr_table_get(r->subprocess_env, name); //table_get
	}

	if(value == NULL){
		char buf[256];
		sprintf(buf, "HTTP_%s", name);
		value = (char *) apr_table_get(r->subprocess_env, buf); //table_get
	}
    /*if(value==NULL){
       value = (char *) table_get(r->headers_in, name); 
	}*/
	return (value == NULL ? NULL : strdup(value));
}

char *get_post_data(request_rec *r, int *clen) {

	int		ret;
	int		length;
	char	*data		= NULL;
	char	*h			= (char *) apr_table_get( r->headers_in, "Content-length" ) ;//table_get

	if(h == NULL) {
		length = 0;
	}else{
		length = atoi(apr_table_get( r->headers_in, "Content-length" ) );//table_get
	}

	if ( length > 0 && (r->method_number == M_POST || r->method_number == M_PUT) ) {

		if ((ret = ap_setup_client_block(r, REQUEST_CHUNKED_ERROR))) {//setup_client_block
			length = 0;
		} else {
	
			if(ap_should_client_block(r)) { //should_client_block

				int remaining = length;
				data = h = (char *) calloc( length+1,sizeof(char));
				if(data) {
					memset(h, 0, length + 1);
					while ( remaining > 0 ) {
						int len_read, len_to_read = remaining;
						len_read = ap_get_client_block( r, h, len_to_read );//get_client_block
						if ( len_read == 0 )	break;
						remaining -= len_read;
						h += len_read;
					}
				}
			}
		}
	}

	*clen = length;

	return data;
}

void write_header(request_rec *r, char *sz_status, char *sz_headers) {
	
	char *ptr = sz_headers;	
	char *perline;
    char *pdest,*pdest1;
	char *header,*value;
    int  result;
	
	if(sz_status != NULL) {
		//r->status = 200;
		r->status_line = sz_status;
	}

	while(ptr){
		
		pdest = strstr(ptr, "\r\n");
		
		if (pdest){
			result = pdest - ptr + 1;

			perline=calloc(result,sizeof(char));
			strncpy(perline,ptr,result);
			perline[result-1]='\0';

			pdest = pdest + 2;
		}

		pdest1 = strstr(perline,":");
		
		if (pdest1){
			result = pdest1 - perline + 1;

			header = calloc(result,sizeof(char));
			strncpy(header,perline,result);
			header[result-1] = '\0';

			value = pdest1 + 2;

			if (!strcasecmp(header,"Content-Type")){
				r->content_type = value;
			}else if (!strcasecmp(header,"Set-Cookie")){
				apr_table_add(r->headers_out, header,value);	
			}else{
				ap_table_setn(r->headers_out, header,value);
			}
		}
		ptr=pdest;
	}
}

int read_bytes(void *bytes, int size, int length, int sock)
{
	int value    = 1;
	int read_len = 0;
	int	ret		 = 0;

	length = size * length;

	while ((read_len < length) && (value > 0)) {
		value = recv(sock, (char *) bytes + read_len, length - read_len, 0);
	    if (value > 0) 	read_len += value;
	}
	
	if(read_len != length) 	ret = -1;
	
	return ret;
}

int send_bytes(void *bytes, int size, int length, int sock){

	int value		= 1;
	int sent_bytes	= 0;
	int avail		= (size * length);

	while ((sent_bytes < avail) && (value > 0)) {
		value = send(sock, (const char *) bytes + sent_bytes, avail - sent_bytes, 0);
		if (value < 0 && getErrno() == EINTR) 
			continue;
		sent_bytes += value;
	}

	if(sent_bytes !=(size * length)){
		return -1;
	}else{
		return sent_bytes;
	}
}

void begin_synch(){

#ifdef WIN32
	EnterCriticalSection(&poolLock);	
#elif defined(NSAPI) 
	crit_enter(poolLock);
#endif 

}

void end_synch(){

#ifdef WIN32
	LeaveCriticalSection(&poolLock);	
#elif defined(NSAPI)
	crit_exit(poolLock);
#endif 
}


void init_synch(){

#ifdef WIN32
	InitializeCriticalSection(&poolLock);
#elif defined(NSAPI) 
	poolLock = crit_init();
#endif 
}

tcpPool * init_pool(char *host, int port, int init, int max, int timeout) {

	int		i;
	tcpPool *tp = (tcpPool *) calloc(1,sizeof(tcpPool));

	tp->size     = init;
	tp->max_size = max;
	tp->timeout  = timeout;
	tp->host     = host;
	tp->port     = port;

    //It will crash if tp->max_size > 1024, Liu Hongdong 7/18/2000.
	//max_size = MaxPoolingSize
	for(i=0;i<tp->max_size;i++) {
		tp->pool[i].sock = -1;
		tp->pool[i].timeout = timeout;
		tp->pool[i].avail = FALSE;
	}

	for(i=0;i<tp->size;i++) {
		int sock = get_connection_socket(host, port);
		if(sock == -1) continue;
		tp->pool[i].sock = sock;
		tp->pool[i].avail = TRUE;
	}

	init_synch();

	return tp;
}

int is_socket_readable(int socket, int secs, int millis) {

    int				ds;
    fd_set			fs;
    struct timeval	tm;

	ds = ((int) socket) + 1;
	tm.tv_sec = secs;
	tm.tv_usec = millis;

	FD_ZERO(&fs);
	FD_SET(socket, &fs);

	return select(ds, &fs, NULL, NULL, &tm);
}

void destroy_tcpPool(tcpPool *tp) {

	int i;

	for(i=0;i<tp->size;i++) {
		if(tp->pool[i].sock != -1) {
			close_connection(tp->pool[i].sock);
		}
	}
}

sockState * get_connection(tcpPool *tp, char *host, int port) {

    int i, sock, index;

	begin_synch();
    
	for(i=0;i<tp->size;i++) {
		if(tp->pool[i].avail == TRUE && tp->pool[i].sock != -1) {
			if(is_socket_readable(tp->pool[i].sock,0,0)){

				closesocket(tp->pool[i].sock);
				tp->pool[i].sock  = -1;
				tp->pool[i].avail = FALSE;

			}else{

				tp->pool[i].avail = FALSE;
				end_synch();
				return &tp->pool[i];

			}
		}
	}


	index = -1;
	for(i=0;i<tp->size;i++)	{
		if(tp->pool[i].sock == -1) {
			index = i;
			break;
		}
	}
	
	if(index == -1 && tp->size >= tp->max_size) {
		end_synch();
		return NULL;
	}

	if((sock = get_connection_socket(host, port)) == -1) {
		end_synch();
		return NULL;
	}

	/*
	index = -1;
	for(i=0;i<tp->size;i++)	{
		if(tp->pool[i].sock == -1) {
			index = i;
			break;
		}
	}

	*/

	if(index == -1) {
		index = tp->size;
		tp->size++;
	}

	tp->pool[index].sock = sock;
	tp->pool[index].avail = FALSE;
	
	end_synch();

	
	return &tp->pool[index];

}


int get_connection_socket(char *host, int port) {

	int					result;
	int					sock;
	int					state = 1;
	int					count = 0;
	struct sockaddr_in	sockaddr;
	BOOL				connection = TRUE;
	
	while(connection) {

		connection = FALSE;
		if ((sock = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP)) < 0) {
			return -1;
		}

		setsockopt(sock, SOL_SOCKET, SO_KEEPALIVE, (char *) &state, sizeof(state));
		setsockopt(sock, IPPROTO_TCP, TCP_NODELAY, (char *) &state, sizeof(state));
		sockaddr.sin_family			= AF_INET;
		sockaddr.sin_addr.s_addr	= inet_addr(host);
		sockaddr.sin_port			= htons((unsigned short) port);
        
		do {
			result = connect(sock, (struct sockaddr *)&sockaddr, sizeof(struct sockaddr_in));
		} while (result == -1 && getErrno() == EINTR);

		if (result == -1) {
			if(sock != -1) 	closesocket(sock);

			if(getErrno() == ETIMEDOUT)	{
				if(count++ < 2)	connection = TRUE;
			}
			
			return -1;
		}
	}

	return sock;
}

void close_connection(int socket) {

	int value = htonl(FINISH);
	send_bytes(&value, sizeof(int), 1, socket);

	closesocket(socket);
}

char * read_data(int f, int *strlen) {

	int		length;
	char	*value;

	read_bytes(&length, sizeof(int), 1, f);

	length = htonl(length);

	value = (char *) calloc(length+1,sizeof(char));

	if(value == NULL) { *strlen = 0; return NULL; }

	read_bytes(value, sizeof(char), length, f);
	value[length] = '\0';

	*strlen = length;

	return value;
}

char * read_string(int f) {

	int		length;
	char	*value;

	read_bytes(&length, sizeof(int), 1, f);

	length = htonl(length);

	value = (char *) calloc(length+1, sizeof(char));

	if(value == NULL) return NULL;

	read_bytes(value, sizeof(char), length, f);
	value[length] = '\0';

	return value;
}

void write_string( char *value, int f) {

	int  length;
	int  len = 0;
	char *h,*end;
	int  *h_int;

	if(value == NULL) {
		length = 0;
	}else{
		len = strlen(value);
		length = htonl(len);
	}

	h = calloc(len+2+1*sizeof(int), sizeof(char));

	h_int = (int *) h;
	h_int[0] = length; 

	end = (char *) &h_int[1];
	end[0] = '\0';
	if (value != NULL) {
		strcat(end, value);
		end = strchr(end, '\0');
	}

	send_bytes(h, sizeof(char), end-h,f);

	free(h);
}

int send_headers(int sock, request_rec *r)
{
	int i, len = 0;
	char *h, *key, *end;
	char *vals[HEADERCOUNT+1];
	int *h_int;
	int command = 1;
	

	for (i=0; i < HEADERCOUNT; i++) {
		key = HEADERNAMES[i];
		if(key=="PATH_TRANSLATED")
		{
			key="REAL_PATH";
    		vals[i] =get_header(key, r);
            key="PATH_TRANSLATED" ;
		}
		else
   		    vals[i] =get_header(key, r);
		len += strlen(key) + 2 + (vals[i] ? strlen(vals[i]) : 0) + 2;
	}


	h = calloc(len+1+2*sizeof(int), sizeof(char));

	if (h == NULL) {
		for (i=0; i < HEADERCOUNT; i++)
			if (vals[i] != NULL)
				free(vals[i]);
		return -1;
	}


	h_int = (int *) h;
	h_int[0] = htonl(command);
	h_int[1] = htonl(len); 

	end = (char *) &h_int[2];
	end[0] = '\0';

	for (i = 0; i < HEADERCOUNT; i++) {
		key = HEADERNAMES[i];
		strcat(end, key);
		strcat(end, ": ");
		strcat(end, (vals[i] == NULL ? "" : vals[i]));
		strcat(end, "\r\n");
		end = strchr(end, '\0');
		if(vals[i] != NULL) 
			free(vals[i]);
	}

	i = send_bytes(h, sizeof(char), end-h, sock);

	free(h);

	return i;
}

void send_post_data(int j, request_rec *r) {

	int length, len;
	char *post_data = get_post_data(r, &length);

	len = htonl(length);

	if(send_bytes(&len, sizeof(int), 1, j) != -1 && length > 0) {

		int left_set = 0;
		int left_send = length;
		int size,result;
		
		while(left_send > 0) {

			if(left_send > 40960) 
				size = 40960;
			else 
				size = left_send;
			if((result = send_bytes(post_data + left_set, sizeof(char), size, j)) == -1)
				break;
			left_send -= result;
			left_set += result;
		}
		free(post_data);
	}
}

Table * get_table(char *filename){

	FILE		*file_ptr;
	char		buffer[1024];
    char		*s, *vptr,*sptr,*tptr;
    int			result;
	Table		*tabs = (Table *) calloc(1,sizeof(Table));

	if(tabs == NULL) return NULL;
	tabs->count = 0;


	if((file_ptr = fopen(filename, "r")) == NULL) return NULL;

	while(!feof(file_ptr)) 	{
		result = fgetc(file_ptr);
		if(result == EOF) break;
		if(result == '#') {
			fgets(buffer, 1024, file_ptr);
		} else if(result == '\r' || result == '\n') {
			continue;
		} else {

			if (tabs->count>=64) break;

			tabs->count++;
			s = tabs->entry[tabs->count-1].key;
			*s++ = result;
			while( (result = fgetc(file_ptr)) != '=' && result != EOF )
				*s++ = result;
			if(result == '=')
				*s = '\0';
			if(result == EOF) {
				fclose(file_ptr);
				return tabs;
			}
			vptr = fgets(tabs->entry[tabs->count-1].val, 4096, file_ptr);
			
			if(vptr == NULL) continue;
			sptr = vptr;
			tptr = sptr;
			while(*sptr != '\0') {
				*sptr = *tptr;
				if(*tptr == '\\' && *(tptr+1) == '\\') {
					tptr+=2;
				}else{
					tptr++;
				}
				sptr++;
			}
			*sptr = '\0';
			if(vptr[strlen(vptr)-1] == '\n' || vptr[strlen(vptr)-1] == '\r' ) vptr[strlen(vptr)-1] = '\0';
		}
	}

	fclose(file_ptr);

	return tabs;
}

int handle_request( sockState *p, request_rec *r, tcpPool *pool) {

	char	*val;
	char	*key;
	char	status[1024];
	int		command = htonl(NOOP);
	int		code;
	int     elapsedTime = htonl(NOOP);

	if (send_headers(p->sock, r) == -1) {
		close_connection(p->sock);
		p->sock = -1;
		return 0;
	}

	command = UNKNOWN;

	memset(status, 0, 1024);

	while( command != FINISH && command > 0 ) {

		if(is_socket_readable(p->sock, p->timeout, 0) == 0) {
			close_connection(p->sock);
			p->sock = -1;
			return -1; 
		}

		if(read_bytes(&command, sizeof(int), 1, p->sock) == -1){

			closesocket( p->sock );
			p->sock = -1;
			if (command == UNKNOWN) {
				return 0;
			}else{
				return -2; 
			}
		}

		command = htonl(command);

		switch(command) {

			case GET_HEADER:

				key = read_string(p->sock);
				val = get_header( key,r);
				write_string( val,p->sock);
				if(key != NULL) free(key);
				if(val != NULL) free(val);
				break;

			case GET_REALPATH:

				key = read_string(p->sock);
				val = get_real_path(r, key);
				write_string( val,p->sock);
				if(key != NULL) free(key);
				if(val != NULL) free(val);
				break;

			case GET_POSTDATA:

				send_post_data(p->sock, r);
				break;

			case PUT_STATUS:

				read_bytes(&code, sizeof(int), 1, p->sock);
				val = read_string(p->sock);
				sprintf(status, "%d %s", htonl(code), val);
				if(val != NULL) free(val);
				break;

			case PUT_HEADER:

				val = read_string(p->sock);
				write_header(r, status, val);
				
				if(val != NULL) free(val);
				break;
				
			case PUT_CLIENT:

				val = read_data(p->sock, &code);
				ap_rwrite(val, code, r);//rwrite
				ap_rflush(r); //rflush
				if(val != NULL) free(val);
				break;
		}

	}

	return 1;
}
