package halcyon.demo;
import halcyon.rjax.*;
import hs.types.*;

public class Proc {
    RJAX rjax = new RJAX();
   	ActiveX proc = null;
	boolean connected = false;

    public Proc() {
	}

    public Proc(String host, int port, String uid, String pwd, String commAdapter) {
        init(host, port, uid, pwd, commAdapter);
    }

    public void init(String host, int port, String uid, String pwd, String commAdapter) {
        try {
            rjax.setHost(host);
            rjax.setPort(port);
            rjax.setUserID(uid);
            rjax.setPassword(pwd);
            rjax.setCommunicationAdapter(commAdapter);
        }
        catch(Exception e) {
            //e.printStackTrace();
        }
    }

    boolean connect() {
    	if(connected)
    		return true;
    	if(!rjax.connect())
    		return false;
    	connected = true;
    	try {
    		proc = rjax.getObject("HSDemo.Process", true);
    		return proc != null;
    	}
    	catch(Exception e) {
    		//e.printStackTrace();
    		disconnect();
    		return false;
    	}
    }

    void disconnect() {
    	if(connected) {
    		if(proc != null) {
    			proc.destroyObject();
    			proc = null;
    		}
    		rjax.disconnect();
    		connected = false;
    	}
    }
    
    public void release() {
    	disconnect();
    }

    public int[] GetProcessIDs() {
    	int[] IDs = null;
    	try {
    		if(!connect())
    			return null;
    		Variant ret = proc.invokeMethod("GetProcessIDs", null, true);
    		int dim[] = new int[1];
    		dim[0] = 0;
    		hs.types.OleArray array = ret.parray();
    		int elements =  (array.getDimension())[0];
    		IDs = new int[elements];
    		for(int i = 0; i < elements; i++) {
    		    dim[0] = i;
        			IDs[i] = ((hs.types.Variant)array.elementAt(dim)).lVal();
    		}
			return IDs;
			
		}
		
		catch(Exception e) {
			//e.printStackTrace();
			disconnect();
		}
		return IDs;
    }

    public String[] GetProcessNames() {
    	String[] names = null;
    	try {
    		if(!connect())
    			return null;
            Variant ret = proc.invokeMethod("GetProcessNames", null, true);
    		hs.types.OleArray array = ret.parray();
    		int elements = (array.getDimension())[0];
    		int dim[] = new int[1];
    		dim[0] = 0;
    		names = new String[elements];
    		for(int i = 0; i < elements; i++) {
    		    dim[0] = i;
        			names[i] = ((hs.types.Variant)array.elementAt(dim)).strVal();
    		}
		}
		catch(Exception e) {
			//e.printStackTrace();
			disconnect();
		}
		return names;
    }

    public int[] GetProcessExecuteTime() {
    	int[] Processtime = null;
    	try {
    		if(!connect())
    			return null;
    		Variant ret = proc.invokeMethod("GetProcessExecuteTime", null, true);
    		hs.types.OleArray array = ret.parray();
    		int elements = (array.getDimension())[0];
    		int dim[] = new int[1];
    		dim[0] = 0;
    		Processtime = new int[elements];
    		for(int i = 0; i < elements; i++) {
    		    dim[0] = i;
        			Processtime[i] = ((hs.types.Variant)array.elementAt(dim)).lVal();
    		}
			return Processtime;
		}
		catch(Exception e) {
			//e.printStackTrace();
			disconnect();
		}
		return Processtime;
    }

    public int[] GetProcessMemorySize() {
    	int[] ProcessSize = null;
    	try {
    		if(!connect())
    			return null;
    		Variant ret = proc.invokeMethod("GetProcessMemorySize", null, true);
    		hs.types.OleArray array = ret.parray();
    		int elements = (array.getDimension())[0];
    		int dim[] = new int[1];
    		ProcessSize = new int[elements];
    		for(int i = 0; i < elements; i++) {
    		    dim[0] = i;
            			ProcessSize[i] = ((hs.types.Variant)array.elementAt(dim)).lVal();
    		}
			return ProcessSize;
		}
		catch(Exception e) {
			//e.printStackTrace();
			disconnect();
		}
		return ProcessSize;
    }

    public boolean Kill(int procID) {
    	try {
    		if(!connect())
    			return false;
    		Variant[] params = { new Variant(procID, false) };
			Variant ret = proc.invokeMethod("Kill", params, true);
			return ret.bool();
		}
		catch(Exception e) {
			//e.printStackTrace();
			disconnect();
		}
		return false;
    }
    
    public static void main(String[] args) {

    	Proc p = null;
    	try {
    		p = new Proc();
    		p.init("qihang",2000,"user","user","halcyon.rjax.comm.TCPClientAdapter");
    		int[] IDs = p.GetProcessIDs();
    		int[] times = p.GetProcessExecuteTime();
    		String[] names = p.GetProcessNames();
    		int[] mems = p.GetProcessMemorySize();
			System.out.println("IDs.length = " + IDs.length + ", times.length = " + times.length + ", names.length = " + names.length +
								", mems.length = " + mems.length);
    		for(int i = 0; i < IDs.length; i++) {
    			System.out.println(String.valueOf(IDs[i]) + ", " + names[i]); // + ", " + times[i] + ", " + mems[i]);
    		}
    		System.out.print("Enter a process ID to kill: ");
            String idStr = (new java.io.DataInputStream(System.in)).readLine();
            p.Kill(Integer.parseInt(idStr));
    	}
    	catch(Exception e) {
    		//e.printStackTrace();
    	}
    	finally {
    		try  {
    			p.release();
    		}
    		catch(Exception e) {
    			//e.printStackTrace();
    		}
    	}
    }
}